﻿using System;
using System.Drawing;

using NAudio.Wave;

namespace TestLibrary
{
    /// <summary>
    /// 웨이브 폼 렌더러
    /// </summary>
    public class WaveFormRenderer
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 렌더링하기 - Render(peakProvider, setting)

        /// <summary>
        /// 렌더링하기
        /// </summary>
        /// <param name="peakProvider">피크 공급자</param>
        /// <param name="setting">설정</param>
        /// <returns>이미지</returns>
        private static Image Render(IPeakProvider peakProvider, WaveFormRendererSetting setting)
        {
            if(setting.DecibelScale)
            {
                peakProvider = new DecibelPeakProvider(peakProvider, 48);
            }

            Bitmap bitmap = new Bitmap(setting.Width, setting.TopHeight + setting.BottomHeight);

            if(setting.BackgroundColor == Color.Transparent)
            {
                bitmap.MakeTransparent();
            }

            using(Graphics graphics = Graphics.FromImage(bitmap))
            {
                graphics.FillRectangle(setting.BackgroundBrush, 0,0,bitmap.Width,bitmap.Height);

                int      middlePoint     = setting.TopHeight;
                int      x               = 0;
                PeakInfo currentPeakInfo = peakProvider.GetNextPeakInfo();

                while(x < setting.Width)
                {
                    PeakInfo nextPeakInfo = peakProvider.GetNextPeakInfo();
                    
                    for(int i = 0; i < setting.PixelCountPerPeak; i++)
                    {
                        float lineHeight = setting.TopHeight * currentPeakInfo.Maximum;

                        graphics.DrawLine(setting.TopPeakPen, x, middlePoint, x, middlePoint - lineHeight);

                        lineHeight = setting.BottomHeight * currentPeakInfo.Minimum;

                        graphics.DrawLine(setting.BottomPeakPen, x, middlePoint, x, middlePoint - lineHeight);

                        x++;
                    }

                    for(int i = 0; i < setting.SpacerPixelCount; i++)
                    {
                        float minimum = Math.Max(currentPeakInfo.Minimum, nextPeakInfo.Minimum);
                        float maximum = Math.Min(currentPeakInfo.Maximum, nextPeakInfo.Maximum);

                        float lineHeight = setting.TopHeight * maximum;

                        graphics.DrawLine(setting.TopSpacerPen, x, middlePoint, x, middlePoint - lineHeight);

                        lineHeight = setting.BottomHeight * minimum;

                        graphics.DrawLine(setting.BottomSpacerPen, x, middlePoint, x, middlePoint - lineHeight); 

                        x++;
                    }

                    currentPeakInfo = nextPeakInfo;
                }
            }

            return bitmap;
        }

        #endregion
        #region 렌더링하기 - Render(waveStream, peakProvider, setting)

        /// <summary>
        /// 렌더링하기
        /// </summary>
        /// <param name="waveStream">웨이브 스트림</param>
        /// <param name="peakProvider">피크 공급자</param>
        /// <param name="setting">설정</param>
        /// <returns>이미지</returns>
        public Image Render(WaveStream waveStream, IPeakProvider peakProvider, WaveFormRendererSetting setting)
        {
            int  byteCountPerSample  = (waveStream.WaveFormat.BitsPerSample / 8);
            long sampleCount         = waveStream.Length / (byteCountPerSample);
            int  sampleCountPerPixel = (int)(sampleCount / setting.Width);
            int  stepSize            = setting.PixelCountPerPeak + setting.SpacerPixelCount;

            peakProvider.Initialize(waveStream.ToSampleProvider(), sampleCountPerPixel * stepSize);

            return Render(peakProvider, setting);
        }

        #endregion
        #region 렌더링하기 - Render(waveStream, setting)

        /// <summary>
        /// 렌더링하기
        /// </summary>
        /// <param name="waveStream">웨이브 스트림</param>
        /// <param name="setting">설정</param>
        /// <returns>이미지</returns>
        public Image Render(WaveStream waveStream, WaveFormRendererSetting setting)
        {
            return Render(waveStream, new MaximumPeakProvider(), setting);
        }

        #endregion
    }
}